import 'package:flutter/gestures.dart';
import 'package:flx_ui/flx_ui.dart';
import 'package:get/get.dart';
import 'package:vibration/vibration.dart';

class TargetController extends GetxController
    with SingleGetTickerProviderMixin {
  // well anim region
  AnimationController? _opacityAnimController;
  AnimationController get opacityAnimController => _opacityAnimController!;

  AnimationController? _radiusAnimController;
  AnimationController get radiusAnimController => _radiusAnimController!;

  Tween<double>? _opacityTween;

  Animation<double>? _opacityAnim;
  Animation<double> get opacityAnim => _opacityAnim!;

  Tween<double>? _radiusTween;

  Animation<double>? _radiusAnim;
  Animation<double> get radiusAnim => _radiusAnim!;

  Offset position = Offset(0, 0);
  bool isMouse = false;

  @override
  void onInit() {
    _initAnims();
    super.onInit();
  }

  void _initAnims() {
    _initShowAnim();
  }

  void _initShowAnim() {
    // controllers
    _opacityAnimController = AnimationController(
      vsync: this,
      duration: Duration(milliseconds: 150),
    );

    _radiusAnimController = AnimationController(
      vsync: this,
      duration: Duration(milliseconds: 250),
    );

    // opacity
    _opacityTween = Tween<double>(
      begin: 0,
      end: 0.5,
    );

    _opacityAnim = _opacityTween!.animate(CurvedAnimation(
      parent: _opacityAnimController!,
      curve: Curves.ease,
    ));

    // radius
    _radiusTween = Tween<double>(
      begin: 30,
      end: 125,
    );

    _radiusAnim = _radiusTween!.animate(CurvedAnimation(
      parent: _radiusAnimController!,
      curve: Curves.easeIn,
    ));
  }

  void showEffect(double? from) {
    opacityAnimController.forward(from: from);
    radiusAnimController.forward(from: from);
  }

  void hideEffect(double? from) {
    opacityAnimController.reverse(from: from);
    radiusAnimController.reverse(from: from);
  }

  void onTap() async {
    if (isMouse) {
      hideEffect(0);
      showEffect(null);
    } else {
      opacityAnimController.forward(from: 0);
      radiusAnimController.forward(from: 0).then((value) {
        opacityAnimController.reverse();
        radiusAnimController.reverse();
      });
    }
  }

  void onEnter(PointerEnterEvent event) {
    isMouse = true;

    position = event.localPosition;

    update();

    showEffect(null);
  }

  void onHover(PointerHoverEvent event) {
    position = event.localPosition;

    update();
  }

  void onExit(PointerExitEvent event) {
    isMouse = false;

    update();

    hideEffect(null);
  }

  void onLongStart(LongPressStartDetails event) async {
    if (isMouse) {
      return;
    }

    position = event.localPosition;

    update();

    await Vibration.vibrate(duration: 50, amplitude: 200);

    showEffect(null);
  }

  void onLongMove(LongPressMoveUpdateDetails event) {
    if (isMouse) {
      return;
    }

    position = event.localPosition;

    update();
  }

  void onLongEnd(LongPressEndDetails event) async {
    if (isMouse) {
      return;
    }

    isMouse = false;

    update();

    hideEffect(null);
  }
}
